<?php
/**
 * @file
 */

namespace Drupal\views_add_button\Plugin\views\area;

use Drupal\views\Plugin\views\area\TokenizeAreaPluginBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\Routing\AdminHtmlRouteProvider;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Defines a views area plugin.
 *
 * @ingroup views_area_handlers
 *
 * @ViewsArea("views_add_button_area")
 */

class ViewsAddButtonArea extends TokenizeAreaPluginBase {
  /**
   * @{inheritdoc}
   */
  public function query() {
    // Leave empty to avoid a query on this field.
  }

  /**
   * Build Bundle Type List
   */
  public function createEntityBundleList() {
    $ret = array();
    foreach(\Drupal::entityManager()->getDefinitions() as $type => $info) {
      // is this a content/front-facing entity?
      if ($info instanceof \Drupal\Core\Entity\ContentEntityType) {
        $label = $info->getLabel();
        if ($label instanceof \Drupal\Core\StringTranslation\TranslatableMarkup) {
          $label = $label->render();
        }
        $ret[$label] = array();
        $bundles = \Drupal::entityManager()->getBundleInfo($type);
        foreach ($bundles as $key => $bundle) {
          if ($bundle['label'] instanceof \Drupal\Core\StringTranslation\TranslatableMarkup) {
            $ret[$label][$type . '+' . $key] = $bundle['label']->render();
          }
          else {
            $ret[$label][$type . '+' . $key] = $bundle['label'];
          }
        }
      }
    }
    return $ret;
  }

  /**
   * Define the available options
   * @return array
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['type'] = array('default' => 'node');
    $options['button_text'] = array('default' => '');
    $options['button_classes'] = array('default' => '');
    $options['query_string'] = array('default' => '');
    $options['destination'] = array('default' => TRUE);
    $options['tokenize'] = array('default' => FALSE, 'bool' => TRUE);
    return $options;
  }

  /**
   * Provide the options form.
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
    $form['type'] = array(
      '#type' => 'select',
      '#title' => t('Entity Type'),
      '#options' => $this ->createEntityBundleList(),
      '#empty_option' => '- Select -',
      '#default_value' => $this->options['type'],
      '#weight' => -10
    );
    $form['button_text'] = array(
      '#type' => 'textfield',
      '#title' => t('Button Text for the add button.'),
      '#default_value' => $this->options['button_text'],
      '#weight' => -8
    );
    $form['query_string'] = array(
      '#type' => 'textfield',
      '#title' => t('Query string to append to the add link'),
      '#default_value' => $this->options['query_string'],
      '#weight' => -8
    );
    $form['button_classes'] = array(
      '#type' => 'textfield',
      '#title' => t('Button classes for the add link - usually "button" or "btn," with additional styling classes.'),
      '#default_value' => $this->options['button_classes'],
      '#weight' => -7
    );
    $form['destination'] = array(
      '#type' => 'checkbox',
      '#title' => t('Include destination parameter?'),
      '#default_value' => $this->options['destination'],
      '#weight' => -6
    );
    $this->tokenForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function render($empty = FALSE)
  {
    // Get the create route
    $type = explode('+', $this->options['type'], 2);
    $entity_type = $type[0];
    $bundle = $type[1];
    $u = $entity_type === $bundle ? '/' . $entity_type . '/add': '/' . $entity_type . '/add/' . $bundle;

    // Build URL Options
    $opts = array();
    $dest = Url::fromRoute('<current>');
    $opts['query']['destination'] = $dest->toString();
    $opts['attributes']['class'] = $this->options['tokenize'] ? $this->tokenizeValue($this->options['button_classes']) : $this->options['button_classes'];
    if ($this->options['query_string']) {
      $q = $this->options['tokenize'] ? $this->tokenizeValue($this->options['query_string']) : $this->options['query_string'];
      if ($q) {
        $qparts = explode('&', $q);
        foreach ($qparts as $part) {
          $p = explode('=', $part);
          if (is_array($p) && count($p) > 1) {
            $opts['query'][$p[0]] = $p[1];
          }
        }
      }
    }

    // Create the link
    $url = Url::fromUserInput($u, $opts);
    $text = $this->options['button_text'] ? $this->options['button_text'] : 'Add ' . $bundle;
    $text = $this->options['tokenize'] ? $this->tokenizeValue($text) : $text;
    $l = Link::fromTextAndUrl(t($text), $url)->toRenderable();

    return $l;
  }
}